@@ -36,9 +36,6 @@ In addition, while many builtin functions are supported, several builtin
 functions that are considered unsafe are missing ('eval', 'exec', and
 'getattr' for example)
 """
-
-from __future__ import division, print_function
-
 import ast
 import time
 import inspect
@@ -46,7 +43,10 @@ from sys import exc_info, stdout, stderr, version_info
 
 from .astutils import (UNSAFE_ATTRS, HAS_NUMPY, make_symbol_table, numpy,
                        op2func, ExceptionHolder, ReturnedNone,
-                       valid_symbol_name, check_pyversion)
+                       valid_symbol_name)
+
+if version_info[0] < 3 or version_info[1] < 5:
+    raise SystemError("Python 3.5 or higher required")
 
 builtins = __builtins__
 if not isinstance(builtins, dict):
@@ -57,12 +57,9 @@ ALL_NODES = ['arg', 'assert', 'assign', 'attribute', 'augassign', 'binop',
              'dict', 'ellipsis', 'excepthandler', 'expr', 'extslice',
              'for', 'functiondef', 'if', 'ifexp', 'index', 'interrupt',
              'list', 'listcomp', 'module', 'name', 'nameconstant', 'num',
-             'pass', 'print', 'raise', 'repr', 'return', 'slice', 'str',
+             'pass', 'raise', 'repr', 'return', 'slice', 'str',
              'subscript', 'try', 'tuple', 'unaryop', 'while', 'constant']
 
-
-PY3 = check_pyversion()
-
 class Interpreter(object):
     """create an asteval Interpreter: a restricted, simplified interpreter
     of mathematical expressions using Python syntax.
@@ -132,11 +129,6 @@ class Interpreter(object):
                 usersyms = {}
             symtable = make_symbol_table(use_numpy=use_numpy, **usersyms)
 
-        if no_print:
-            symtable['print'] = self._nullprinter
-        else:
-            symtable['print'] = self._printer
-
         self.symtable = symtable
         self._interrupt = None
         self.error = []
@@ -147,6 +139,9 @@ class Interpreter(object):
         self.start_time = time.time()
         self.use_numpy = HAS_NUMPY and use_numpy
 
+        symtable['print'] = self._printer
+        self.no_print = no_print or minimal
+
         nodes = ALL_NODES[:]
 
         if minimal or no_if:
@@ -167,8 +162,6 @@ class Interpreter(object):
             nodes.remove('delete')
         if minimal or no_raise:
             nodes.remove('raise')
-        if minimal or no_print:
-            nodes.remove('print')
         if minimal or no_listcomp:
             nodes.remove('listcomp')
         if minimal or no_augassign:
@@ -306,7 +299,10 @@ class Interpreter(object):
             return ret
         except:
             if with_raise:
-                self.raise_exception(node, expr=expr)
+                if len(self.error) == 0:
+                    # Unhandled exception that didn't go through raise_exception
+                    self.raise_exception(node, expr=expr)
+                raise
 
     def __call__(self, expr, **kw):
         """Call class instance as function."""
@@ -613,26 +609,10 @@ class Interpreter(object):
                 out = out and r
         return out
 
-
-    def on_print(self, node):    # ('dest', 'values', 'nl')
-        """Note: implements Python2 style print statement, not print()
-        function.
-        May need improvement....
-        """
-        dest = self.run(node.dest) or self.writer
-        end = ''
-        if node.nl:
-            end = '\n'
-        out = [self.run(tnode) for tnode in node.values]
-        if out and len(self.error) == 0:
-            self._printer(*out, file=dest, end=end)
-
-    def _nullprinter(self, *out, **kws):
-        """swallow print calls"""
-        pass
-
     def _printer(self, *out, **kws):
         """Generic print function."""
+        if self.no_print:
+            return
         flush = kws.pop('flush', True)
         fileh = kws.pop('file', self.writer)
         sep = kws.pop('sep', ' ')
@@ -736,12 +716,8 @@ class Interpreter(object):
 
     def on_raise(self, node):    # ('type', 'inst', 'tback')
         """Raise statement: note difference for python 2 and 3."""
-        if PY3:
-            excnode = node.exc
-            msgnode = node.cause
-        else:
-            excnode = node.type
-            msgnode = node.inst
+        excnode = node.exc
+        msgnode = node.cause
         out = self.run(excnode)
         msg = ' '.join(out.args)
         msg2 = self.run(msgnode)
@@ -763,7 +739,7 @@ class Interpreter(object):
             args = args + self.run(starargs)
 
         keywords = {}
-        if PY3 and func == print:
+        if func == print:
             keywords['file'] = self.writer
 
         for key in node.keywords:
@@ -779,9 +755,10 @@ class Interpreter(object):
         try:
             return func(*args, **keywords)
         except Exception as ex:
+            func_name = getattr(func, '__name__', str(func))
             self.raise_exception(
                 node, msg="Error running function call '%s' with args %s and "
-                "kwargs %s: %s" % (func.__name__, args, keywords, ex))
+                "kwargs %s: %s" % (func_name, args, keywords, ex))
 
     def on_arg(self, node):    # ('test', 'msg')
         """Arg for function definitions."""
@@ -804,10 +781,7 @@ class Interpreter(object):
             keyval = self.run(node.args.args[idef+offset])
             kwargs.append((keyval, defval))
 
-        if PY3:
-            args = [tnode.arg for tnode in node.args.args[:offset]]
-        else:
-            args = [tnode.id for tnode in node.args.args[:offset]]
+        args = [tnode.arg for tnode in node.args.args[:offset]]
         doc = None
         nb0 = node.body[0]
         if isinstance(nb0, ast.Expr) and isinstance(nb0.value, ast.Str):
@@ -815,9 +789,9 @@ class Interpreter(object):
 
         varkws = node.args.kwarg
         vararg = node.args.vararg
-        if PY3 and isinstance(vararg, ast.arg):
+        if isinstance(vararg, ast.arg):
             vararg = vararg.arg
-        if PY3 and isinstance(varkws, ast.arg):
+        if isinstance(varkws, ast.arg):
             varkws = varkws.arg
 
         self.symtable[node.name] = Procedure(node.name, self, doc=doc,
